import asyncio
import json
from asyncio import StreamReader, StreamWriter

from tanks.common import Tank, Direction, Missile
from tanks.ui import UI


class Client:
    def __init__(self, ui: UI, reader_stream: StreamReader, writer_stream: StreamWriter) -> None:
        self.ui = ui
        self.reader_stream = reader_stream
        self.writer_stream = writer_stream
        self.tanks: list[Tank] = []
        self.missiles: list[Missile] = []

    async def handle_user_input(self):
        loop = asyncio.get_running_loop()
        while True:
            user_input = await loop.run_in_executor(executor=None, func=self.ui.get_key)
            await self.send_data_to_server(user_input)
            if user_input == "q":
                return

    async def send_data_to_server(self, data: str) -> None:
        self.writer_stream.write(data.encode())
        await self.writer_stream.drain()

    async def handle_data_from_server(self) -> None:
        while True:
            message_from_server = await self.reader_stream.read(1000)
            gameplay_data = json.loads(message_from_server.decode())
            tanks_data = gameplay_data["tanks"]
            missiles_data = gameplay_data["missiles"]
            self.tanks = [Tank(**tank_data) for tank_data in tanks_data]
            self.missiles = [Missile(**missile_data) for missile_data in missiles_data]
            self.ui.draw_ui(self.tanks, self.missiles)
