import curses
from typing import Protocol

from tanks.common import Direction, Tank, GAME_AREA_HEIGHT, GAME_AREA_WIDTH, Missile


class UI(Protocol):
    def draw_ui(self, tanks: list[Tank], missiles: list[Missile]) -> None:
        ...

    def get_key(self) -> str:
        ...


class CursesUI:
    def __init__(self) -> None:
        self.window = curses.initscr()
        curses.noecho()
        curses.cbreak()
        curses.start_color()
        self.window.clear()
        self.window.refresh()

    def draw_ui(self, tanks: list[Tank], missiles: list[Missile]) -> None:
        self.window.clear()
        self.draw_area_borders()
        for tank in tanks:
            self.draw_tank(tank)
        for missile in missiles:
            self.draw_missile(missile)
        self.window.move(0, 0)
        self.window.refresh()

    def draw_area_borders(self):
        self.window.addstr(0, 0, GAME_AREA_WIDTH * "#")
        self.window.addstr(GAME_AREA_HEIGHT, 0, GAME_AREA_WIDTH * "#")
        for pos_y in range(GAME_AREA_HEIGHT):
            self.window.addstr(pos_y, 0, "#")
            self.window.addstr(pos_y, GAME_AREA_WIDTH, "#")

    def draw_tank(self, tank: Tank):
        self.window.addstr(tank.center_y - 1, tank.center_x, "-")
        self.window.addstr(tank.center_y + 1, tank.center_x, "-")
        self.window.addstr(tank.center_y, tank.center_x - 1, "|")
        self.window.addstr(tank.center_y, tank.center_x + 1, "|")

        if tank.direction == Direction.UP:
            self.window.addstr(tank.center_y - 1, tank.center_x, "^")
        elif tank.direction == Direction.DOWN:
            self.window.addstr(tank.center_y + 1, tank.center_x, "˅")
        elif tank.direction == Direction.LEFT:
            self.window.addstr(tank.center_y, tank.center_x - 2, "<")
        elif tank.direction == Direction.RIGHT:
            self.window.addstr(tank.center_y, tank.center_x + 2, ">")

    def draw_missile(self, missile: Missile) -> None:
        if missile.direction == Direction.LEFT or missile.direction == Direction.RIGHT:
            self.window.addstr(missile.center_y, missile.center_x, "-")
        else:
            self.window.addstr(missile.center_y, missile.center_x, "|")

    def get_key(self):
        return self.window.getkey()
